@@ -9,7 +9,7 @@
 
 """
 
-from __future__ import division
+
 from collections import OrderedDict
 
 import numpy as np
@@ -41,7 +41,7 @@ class AccumulatorModel(kabuki.Hierarchical):
 
 
     def _create_an_average_model(self):
-        raise NotImplementedError, "This method has to be overloaded. See HDDMBase."
+        raise NotImplementedError("This method has to be overloaded. See HDDMBase.")
 
 
     def _quantiles_optimization(self, method, quantiles=(.1, .3, .5, .7, .9 ), n_runs=3):
@@ -106,7 +106,7 @@ class AccumulatorModel(kabuki.Hierarchical):
             #set average quantiles  to have the same statitics
             obs_knode = [x for x in self.knodes if x.name == 'wfpt'][0]
             node_name = obs_knode.create_node_name(tag) #get node name
-            average_node = average_model.nodes_db.ix[node_name]['node'] #get the average node
+            average_node = average_model.nodes_db.loc[node_name]['node'] #get the average node
             average_node.set_quantiles_stats(quantiles, n_samples, emp_rt, freq_obs, p_upper) #set the quantiles
 
         return average_model
@@ -147,7 +147,7 @@ class AccumulatorModel(kabuki.Hierarchical):
             return results
 
         #init DataFrame to save results
-        res =  pd.DataFrame(np.zeros((n_bootstraps, len(self.values))), columns=self.values.keys())
+        res =  pd.DataFrame(np.zeros((n_bootstraps, len(self.values))), columns=list(self.values.keys()))
 
         #prepare view for parallelization
         if parallel_profile is not None: #create view
@@ -163,8 +163,8 @@ class AccumulatorModel(kabuki.Hierarchical):
                              method=method, quantiles=quantiles, n_runs=n_runs):
 
             #resample data
-            new_data = data.ix[np.random.randint(0, len(data), len(data))]
-            new_data = new_data.set_index(pd.Index(range(len(data))))
+            new_data = data.iloc[np.random.randint(0, len(data), len(data))]
+            new_data = new_data.set_index(pd.Index(list(range(len(data)))))
             h = accumulator_class(new_data, **class_kwargs)
 
             #run optimization
@@ -173,9 +173,9 @@ class AccumulatorModel(kabuki.Hierarchical):
             return pd.Series(h.values, dtype=np.float)
 
         #bootstrap iterations
-        for i_strap in xrange(n_bootstraps):
+        for i_strap in range(n_bootstraps):
             if view is None:
-                res.ix[i_strap] = single_bootstrap(self.data)
+                res.iloc[i_strap] = single_bootstrap(self.data)
             else:
                 # append to job queue
                 runs_list[i_strap] = view.apply_async(single_bootstrap, self.data)
@@ -183,13 +183,13 @@ class AccumulatorModel(kabuki.Hierarchical):
         #get parallel results
         if view is not None:
             view.wait(runs_list)
-            for i_strap in xrange(n_bootstraps):
-                res.ix[i_strap] = runs_list[i_strap].get()
+            for i_strap in range(n_bootstraps):
+                res.iloc[i_strap] = runs_list[i_strap].get()
 
         #get statistics
         stats = res.describe()
         for q in [2.5, 97.5]:
-            stats = stats.append(pd.DataFrame(res.quantile(q/100.), columns=[`q` + '%']).T)
+            stats = stats.append(pd.DataFrame(res.quantile(q/100.), columns=[repr(q) + '%']).T)
 
         self.bootstrap_stats = stats.sort_index()
         return results
@@ -200,7 +200,7 @@ class AccumulatorModel(kabuki.Hierarchical):
 
         if method == 'ML':
             if self.is_group_model:
-                raise TypeError, "optimization method is not defined for group models"
+                raise TypeError("optimization method is not defined for group models")
             else:
                 results, _ = self._optimization_single(method, quantiles, n_runs=n_runs)
                 return results
@@ -271,20 +271,20 @@ class AccumulatorModel(kabuki.Hierarchical):
         all_results = []
         values = original_values.copy()
         inf_objective = False
-        for i_run in xrange(n_runs):
+        for i_run in range(n_runs):
             #initalize values to a random point
             values_iter = 0
             while inf_objective:
                 values_iter += 1
                 values = original_values + np.random.randn(len(values))*(2**-values_iter)
-                self.set_values(dict(zip(names, values)))
+                self.set_values(dict(list(zip(names, values))))
                 inf_objective = np.isinf(objective(values))
 
             #optimze
             try:
-                res_tuple = fmin_powell(objective, values, full_output=True)
+                res_tuple = fmin_powell(objective, values, full_output=True, maxiter=100, maxfun=50000)
             except Exception:
-                res_tuple = fmin(objective, values, full_output=True)
+                res_tuple = fmin(objective, values, full_output=True, maxiter=100, maxfun=50000)
             all_results.append(res_tuple)
 
             #reset inf_objective so values be resampled
@@ -293,7 +293,7 @@ class AccumulatorModel(kabuki.Hierarchical):
         #get best results
         best_idx = np.nanargmin([x[1] for x in all_results])
         best_values = all_results[best_idx][0]
-        self.set_values(dict(zip(names, best_values)))
+        self.set_values(dict(list(zip(names, best_values))))
         results = self.values
 
         #calc BIC for G^2
@@ -406,6 +406,46 @@ class AccumulatorModel(kabuki.Hierarchical):
 
         return knodes
 
+    def _create_family_normal_non_centered(self, name, value=0, g_mu=None,
+                             g_tau=15**-2, std_lower=1e-10,
+                             std_upper=100, std_value=.1):
+        """Similar to _create_family_normal() but using a non-centered 
+        approach to estimating individual differences.
+
+        See _create_family_normal() help for more information.
+
+        """
+        if g_mu is None:
+            g_mu = value
+
+        knodes = OrderedDict()
+
+        if self.is_group_model and name not in self.group_only_nodes:
+            g = Knode(pm.Normal, '%s' % name, mu=g_mu, tau=g_tau,
+                      value=value, depends=self.depends[name])
+            depends_std = self.depends[name] if self.std_depends else ()
+            std = Knode(pm.Uniform, '%s_std' % name, lower=std_lower,
+                        upper=std_upper, value=std_value, depends=depends_std)
+            offset_subj = Knode(pm.Normal, '%s_offset_subj' % name, mu=0, tau=5**-2,
+                                value=0, depends=('subj_idx',),
+                                subj=True, hidden=True, plot=False)
+            subj = Knode(pm.Deterministic, '%s_subj'%name, eval=lambda x,y,z: x+y*z,
+                         x=g,y=offset_subj,z=std,
+                         depends=('subj_idx',), plot=self.plot_subjs,
+                         trace=True, hidden=False, subj=True)
+            knodes['%s'%name] = g
+            knodes['%s_std'%name] = std
+            knodes['%s_offset_subj'%name] = offset_subj
+            knodes['%s_bottom'%name] = subj
+
+        else:
+            subj = Knode(pm.Normal, name, mu=g_mu, tau=g_tau,
+                         value=value, depends=self.depends[name])
+
+            knodes['%s_bottom'%name] = subj
+
+        return knodes
+        
     def _create_family_invlogit(self, name, value, g_mu=None, g_tau=15**-2,
                                std_std=0.2, std_value=.1):
         """Similar to _create_family_normal_normal_hnormal() but adds a invlogit
@@ -645,7 +685,7 @@ class HDDMBase(AccumulatorModel):
     """
 
     def __init__(self, data, bias=False, include=(),
-                 wiener_params=None, p_outlier=0., **kwargs):
+                 wiener_params=None, p_outlier=0.05, **kwargs):
 
         self.default_intervars = kwargs.pop('default_intervars', {'sz': 0, 'st': 0, 'sv': 0})
 
@@ -663,8 +703,8 @@ class HDDMBase(AccumulatorModel):
         if bias:
             self.include.add('z')
 
-        possible_parameters = ('v', 'a', 't', 'z', 'st', 'sz', 'sv', 'p_outlier')
-        assert self.include.issubset(possible_parameters), """Received and invalid parameter using the 'include' keyword.
+        possible_parameters = ('v', 'a', 't', 'z', 'st', 'sz', 'sv', 'p_outlier', 'alpha')
+        assert self.include.issubset(possible_parameters), """Received an invalid parameter using the 'include' keyword.
         parameters received: %s
         parameters allowed: %s """ % (tuple(self.include), possible_parameters)
 
@@ -720,7 +760,7 @@ class HDDMBase(AccumulatorModel):
         knodes = self._create_stochastic_knodes(self.include)
         knodes['wfpt'] = self._create_wfpt_knode(knodes)
 
-        return knodes.values()
+        return list(knodes.values())
 
     def plot_posterior_predictive(self, *args, **kwargs):
         if 'value_range' not in kwargs:
@@ -779,7 +819,7 @@ class HDDMBase(AccumulatorModel):
         #this code only check that the arguments are as expected, i.e. the constructor was not change
         #since we wrote this function
         init_args = set(inspect.getargspec(self.__init__).args)
-        known_args = set(['wiener_params', 'include', 'self', 'bias', 'data', 'p_outlier'])
+        known_args = set(['wiener_params', 'include', 'self', 'bias', 'data','p_outlier'])
         assert known_args.issuperset(init_args), "Arguments of the constructor are not as expected"
 
         #create the avg model
